#!/usr/bin/env node

// src/index.ts
import { cac } from "cac";
import chalk from "chalk";
import cpy from "cpy";
import { execa } from "execa";
import fs from "fs-extra";
import prompts from "prompts";

// package.json
var name = "create-wagmi";
var version = "0.0.0-20221027075654";

// src/templates.ts
var templates = [
  {
    name: "next",
    title: "Next.js",
    description: "Next.js wagmi project"
  },
  {
    name: "next-with-connectkit",
    title: "Next.js + ConnectKit",
    description: "Next.js wagmi project with ConnectKit included"
  },
  {
    name: "next-with-rainbowkit",
    title: "Next.js + RainbowKit",
    description: "Next.js wagmi project with RainbowKit included"
  },
  {
    name: "vite-react",
    title: "Vite",
    description: "Vite React wagmi project"
  },
  {
    name: "create-react-app",
    title: "Create React App",
    description: "Create React App wagmi project"
  }
];

// src/utils/detectPackageManager.ts
import { detect } from "detect-package-manager";
var detectPackageManager = () => {
  const userAgent = process.env.npm_config_user_agent;
  if (userAgent) {
    if (userAgent.includes("pnpm"))
      return "pnpm";
    if (userAgent.includes("npm"))
      return "npm";
    if (userAgent.includes("yarn"))
      return "yarn";
  }
  return detect();
};

// src/utils/validatePackageName.ts
import validateNpmPackageName from "validate-npm-package-name";
var validatePackageName = (projectName) => {
  const { validForNewPackages, warnings } = validateNpmPackageName(projectName);
  return {
    valid: validForNewPackages,
    warnings
  };
};

// src/index.ts
import path from "path";
import { fileURLToPath } from "url";
var CLIError = class extends Error {
};
var log = console.log;
var cli = cac(name).version(version).usage(`${chalk.green("<project-directory>")} [options]`).option(
  "-t, --template [name]",
  `A template to bootstrap with. Available: ${templates.map(({ name: name2 }) => name2).join(", ")}`
).option("--npm", "Use npm as your package manager").option("--pnpm", "Use pnpm as your package manager").option("--yarn", "Use yarn as your package manager").option("--skip-git", "Skips initializing the project as a git repository").help();
void (async () => {
  var _a;
  try {
    const { args, options } = cli.parse(process.argv);
    if (options.help)
      return;
    const __dirname = fileURLToPath(new URL(".", import.meta.url));
    const templatesPath = path.join(__dirname, "..", "templates");
    let templateName = options.template || options.t;
    if (templateName) {
      const templatePath2 = path.join(templatesPath, templateName);
      if (!fs.existsSync(templatePath2))
        throw new CLIError(
          [
            chalk.red(`\u{1F648} the template "${templateName}" does not exist.`),
            `\u{1F449} choose a valid name. Available: ${templates.map(({ name: name2 }) => name2).join(", ")}`
          ].join("\n")
        );
    }
    log();
    log(chalk.magenta("\u2665 gm, welcome to wagmi \u2665"));
    log();
    let projectPath;
    let projectName;
    if (args[0]) {
      projectPath = args[0];
      const splitPath = args[0].split("/");
      projectName = splitPath[splitPath.length - 1];
    }
    if (!projectName) {
      projectName = (await prompts({
        initial: "my-wagmi-app",
        name: "projectName",
        message: "What would you like to name your project?",
        type: "text",
        validate: (name2) => !validatePackageName(name2).valid ? `"${name2}" is not a valid project name. Enter another name.` : true
      })).projectName;
      if (!projectName)
        throw new CLIError();
      projectPath = projectName;
      log(chalk.cyan("\u{1F44D} Sick name"));
      log();
    }
    if (!validatePackageName(projectName).valid)
      throw new CLIError(
        [
          chalk.red(`\u{1F648} "${projectName}" is not a valid project name.`),
          (_a = validatePackageName(projectName).warnings) == null ? void 0 : _a.map(
            (warning) => `\u{1F449} ${warning}`
          )
        ].join("\n")
      );
    const targetPath = path.join(process.cwd(), projectPath);
    if (fs.existsSync(targetPath))
      throw new CLIError(
        [
          chalk.red(`\u{1F648} the directory "${projectPath}" already exists.`),
          `\u{1F449} choose another name or delete the directory.`
        ].join("\n")
      );
    if (!templateName) {
      templateName = (await prompts({
        name: "templateName",
        message: "What template would you like to use?",
        type: "select",
        choices: templates.map(({ description, name: name2, title }) => ({
          description,
          title,
          value: name2
        }))
      })).templateName;
    }
    log(chalk.cyan("\u{1F477}\u200D\u2642\uFE0F Creating a new wagmi app in", chalk.green(targetPath)));
    log();
    const templatePath = path.join(templatesPath, templateName);
    await cpy(path.join(templatePath, "**", "*"), targetPath, {
      rename: (name2) => name2.replace(/^_dot_/, ".")
    });
    const packageJson = await fs.readJSON(path.join(targetPath, "package.json"));
    packageJson.name = projectName;
    await fs.writeFile(
      path.join(targetPath, "package.json"),
      JSON.stringify(packageJson, null, 2)
    );
    const packageManager = options.pnpm ? "pnpm" : options.yarn ? "yarn" : options.npm ? "npm" : await detectPackageManager();
    log(
      chalk.cyan(
        `\u{1F4E6} Installing dependencies with ${chalk.bold(
          packageManager
        )}. This may take a minute or so\u2026`
      )
    );
    log();
    await execa(packageManager, ["install"], {
      cwd: targetPath,
      stdio: "inherit"
    });
    if (!options.skipGit) {
      await execa("git", ["init"], { cwd: targetPath });
      await execa("git", ["add", "."], { cwd: targetPath });
      await execa(
        "git",
        [
          "commit",
          "--no-verify",
          "--message",
          "Initial commit from create-wagmi"
        ],
        { cwd: targetPath }
      );
    }
    log();
    log(chalk.green(`\u{1F525} Your wagmi app was set up!`));
    log();
    log(
      chalk.cyan(
        `\u{1F680} To start your app, run \`${chalk.bold(
          `cd ${projectPath}`
        )}\` and then \`${chalk.bold(
          `${packageManager}${packageManager === "npm" ? " run" : ""} dev`
        )}\``
      )
    );
    log();
    log(chalk.magenta("\u2665 gn \u2665"));
  } catch (error) {
    log(
      error instanceof CLIError ? error.message : chalk.red(error.message)
    );
    process.exit(1);
  }
})();
